from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import from_int, to_enum
from eksisozluk.Models.User.KarmaName import KarmaName


@dataclass
class Karma:
    name: KarmaName
    value: int

    @staticmethod
    def from_dict(obj: Any) -> 'Karma':
        assert isinstance(obj, dict)
        name = KarmaName(obj.get("Name"))
        value = from_int(obj.get("Value"))
        return Karma(name, value)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Name"] = to_enum(KarmaName, self.name)
        result["Value"] = from_int(self.value)
        return result


from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import from_none, from_bool


@dataclass
class Video:
    display_info: None
    in_topic_video: bool

    @staticmethod
    def from_dict(obj: Any) -> 'Video':
        assert isinstance(obj, dict)
        display_info = from_none(obj.get("DisplayInfo"))
        in_topic_video = from_bool(obj.get("InTopicVideo"))
        return Video(display_info, in_topic_video)

    def to_dict(self) -> dict:
        result: dict = {}
        result["DisplayInfo"] = from_none(self.display_info)
        result["InTopicVideo"] = from_bool(self.in_topic_video)
        return result


from eksisozluk.Models.User.User import User
from eksisozluk.Models.__init__ import *


@dataclass
class UserResponse:
    success: bool
    message: None
    user: User

    @staticmethod
    def from_dict(obj: Any) -> 'UserResponse':
        assert isinstance(obj, dict)
        success = from_bool(obj.get("Success"))
        message = from_none(obj.get("Message"))
        data = User.from_dict(obj.get("Data"))
        return UserResponse(success, message, data)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Success"] = from_bool(self.success)
        result["Message"] = from_none(self.message)
        result["Data"] = to_class(User, self.user)
        return result


def user_response_from_dict(s: Any) -> List[UserResponse]:
    return from_list(UserResponse.from_dict, s)


def user_response_to_dict(x: List[UserResponse]) -> Any:
    return from_list(lambda x: to_class(UserResponse, x), x)



from enum import Enum


class BadgeName(Enum):
    AZIMLI = "azimli"
    BOT = "bot"
    CIZRELI = "cizreli"
    CPU = "cpu"
    EDITÖR = "editör"
    EKŞI = "ekşi"
    EKŞITEKER = "ekşiteker"
    EMEKLI = "emekli"
    GUDIK = "gudik"
    ITFAIYECI = "itfaiyeci"
    KARAGÖZ = "karagöz"
    KAYIP = "kayıp"
    LANETLI = "lanetli"
    LEYLA = "leyla"
    MERHUM = "merhum"
    MÜCAHIT = "mücahit"
    ORGANIZATÖR = "organizatör"
    ORMANCI = "ormancı"
    PRAETOR = "praetor"
    SCENER = "scener"
    TASNIFÇI = "tasnifçi"
    THE_64_BIT = "64-bit"
    ÇAYLAK = "çaylak"


from eksisozluk.Models.__init__ import *
from eksisozluk.Models.Auth.EksiToken import EksiToken


@dataclass
class LoginResponse:
    success: Optional[bool]
    data: EksiToken
    message: Optional[str] = None

    @staticmethod
    def from_dict(obj: Any) -> 'LoginResponse':
        assert isinstance(obj, dict)
        success = from_union([from_bool, from_none], obj.get("Success"))
        message = from_union([from_str, from_none], obj.get("Message"))
        data = EksiToken.from_dict(obj.get("Data"))
        return LoginResponse(success, data, message)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Success"] = from_union([from_bool, from_none], self.success)
        result["Message"] = from_union([from_str, from_none], self.message)
        result["Data"] = to_class(EksiToken, self.data)
        return result


def login_response_from_dict(s: Any) -> LoginResponse:
    return LoginResponse.from_dict(s)


def login_response_to_dict(x: LoginResponse) -> Any:
    return to_class(LoginResponse, x)



from dataclasses import dataclass
from typing import List, Optional, Any

from eksisozluk.Models import from_list, from_bool, from_int, from_union, from_none, from_str, to_class
from eksisozluk.Models.User.Badge import Badge
from eksisozluk.Models.Topic.Topic import Topic
from eksisozluk.Models.User.UserInfo import UserInfo


@dataclass
class User:
    user_info: UserInfo
    badges: List[Badge]
    has_entry_used_on_seyler: bool
    follower_count: int
    followings_count: int
    picture: Optional[str] = None
    pinned_entry: Optional[Topic] = None

    @staticmethod
    def from_dict(obj: Any) -> 'User':
        assert isinstance(obj, dict)
        user_info = UserInfo.from_dict(obj.get("UserInfo"))
        badges = from_list(Badge.from_dict, obj.get("Badges"))
        has_entry_used_on_seyler = from_bool(obj.get("HasEntryUsedOnSeyler"))
        follower_count = from_int(obj.get("FollowerCount"))
        followings_count = from_int(obj.get("FollowingsCount"))
        picture = from_union([from_none, from_str], obj.get("Picture"))
        pinned_entry = from_union([from_none, Topic.from_dict], obj.get("PinnedEntry"))
        return User(user_info, badges, has_entry_used_on_seyler, follower_count, followings_count, picture, pinned_entry)

    def to_dict(self) -> dict:
        result: dict = {}
        result["UserInfo"] = to_class(UserInfo, self.user_info)
        result["Badges"] = from_list(lambda x: to_class(Badge, x), self.badges)
        result["HasEntryUsedOnSeyler"] = from_bool(self.has_entry_used_on_seyler)
        result["FollowerCount"] = from_int(self.follower_count)
        result["FollowingsCount"] = from_int(self.followings_count)
        result["Picture"] = from_union([from_none, from_str], self.picture)
        result["PinnedEntry"] = from_union([from_none, lambda x: to_class(Topic, x)], self.pinned_entry)
        return result


from eksisozluk.Models.Index.Index import Index
from eksisozluk.Models.__init__ import *


@dataclass
class IndexResponse:
    success: bool
    message: None
    index: Index

    @staticmethod
    def from_dict(obj: Any) -> 'IndexResponse':
        assert isinstance(obj, dict)
        success = from_bool(obj.get("Success"))
        message = from_none(obj.get("Message"))
        data = Index.from_dict(obj.get("Data"))
        return IndexResponse(success, message, data)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Success"] = from_bool(self.success)
        result["Message"] = from_none(self.message)
        result["Data"] = to_class(Index, self.index)
        return result


def index_response_from_dict(s: Any) -> IndexResponse:
    return IndexResponse.from_dict(s)


def index_response_to_dict(x: IndexResponse) -> Any:
    return to_class(IndexResponse, x)



from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import from_int


@dataclass
class UserInfoEntryCounts:
    total: int
    last_month: int
    last_week: int
    today: int

    @staticmethod
    def from_dict(obj: Any) -> 'UserInfoEntryCounts':
        assert isinstance(obj, dict)
        total = from_int(obj.get("Total"))
        last_month = from_int(obj.get("LastMonth"))
        last_week = from_int(obj.get("LastWeek"))
        today = from_int(obj.get("Today"))
        return UserInfoEntryCounts(total, last_month, last_week, today)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Total"] = from_int(self.total)
        result["LastMonth"] = from_int(self.last_month)
        result["LastWeek"] = from_int(self.last_week)
        result["Today"] = from_int(self.today)
        return result


from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import from_str, to_enum
from eksisozluk.Models.User.BadgeName import BadgeName


@dataclass
class Badge:
    name: BadgeName
    description: str

    @staticmethod
    def from_dict(obj: Any) -> 'Badge':
        assert isinstance(obj, dict)
        name = BadgeName(obj.get("Name"))
        description = from_str(obj.get("Description"))
        return Badge(name, description)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Name"] = to_enum(BadgeName, self.name)
        result["Description"] = from_str(self.description)
        return result


from dataclasses import dataclass
from datetime import datetime
from typing import Optional, Any

from eksisozluk.Models import from_int, from_datetime, from_bool, from_none, from_union, from_str, to_class
from eksisozluk.Models.User.CursePeriod import CursePeriod
from eksisozluk.Models.User.Karma import Karma
from eksisozluk.Models.User.UserIdentifier import UserIdentifier
from eksisozluk.Models.User.UserInfoEntryCounts import UserInfoEntryCounts


@dataclass
class UserInfo:
    user_identifier: UserIdentifier
    remaining_invitation: int
    entry_counts: UserInfoEntryCounts
    last_entry_date: datetime
    standing_queue_number: int
    has_any_pending_invitation: bool
    is_buddy: bool
    is_blocked: bool
    is_followed: bool
    is_corporate: bool
    is_deactivated: bool
    is_karma_shown: bool
    is_caylak: bool
    is_index_titles_blocked: bool
    note: None
    is_cursed: bool
    is_banned: bool
    display_twitter_profile: bool
    display_facebook_profile: bool
    display_instagram_profile: bool
    twitter_screen_name: Optional[str] = None
    facebook_profile_url: Optional[str] = None
    facebook_screen_name: Optional[str] = None
    instagram_screen_name: Optional[str] = None
    instagram_profile_url: Optional[str] = None
    karma: Optional[Karma] = None
    curse_period: Optional[CursePeriod] = None

    @staticmethod
    def from_dict(obj: Any) -> 'UserInfo':
        assert isinstance(obj, dict)
        user_identifier = UserIdentifier.from_dict(obj.get("UserIdentifier"))
        remaining_invitation = from_int(obj.get("RemainingInvitation"))
        entry_counts = UserInfoEntryCounts.from_dict(obj.get("EntryCounts"))
        last_entry_date = from_datetime(obj.get("LastEntryDate"))
        standing_queue_number = from_int(obj.get("StandingQueueNumber"))
        has_any_pending_invitation = from_bool(obj.get("HasAnyPendingInvitation"))
        is_buddy = from_bool(obj.get("IsBuddy"))
        is_blocked = from_bool(obj.get("IsBlocked"))
        is_followed = from_bool(obj.get("IsFollowed"))
        is_corporate = from_bool(obj.get("IsCorporate"))
        is_deactivated = from_bool(obj.get("IsDeactivated"))
        is_karma_shown = from_bool(obj.get("IsKarmaShown"))
        is_caylak = from_bool(obj.get("IsCaylak"))
        is_index_titles_blocked = from_bool(obj.get("IsIndexTitlesBlocked"))
        note = from_none(obj.get("Note"))
        is_cursed = from_bool(obj.get("IsCursed"))
        is_banned = from_bool(obj.get("IsBanned"))
        display_twitter_profile = from_bool(obj.get("DisplayTwitterProfile"))
        display_facebook_profile = from_bool(obj.get("DisplayFacebookProfile"))
        display_instagram_profile = from_bool(obj.get("DisplayInstagramProfile"))
        twitter_screen_name = from_union([from_none, from_str], obj.get("TwitterScreenName"))
        facebook_profile_url = from_union([from_none, from_str], obj.get("FacebookProfileUrl"))
        facebook_screen_name = from_union([from_none, from_str], obj.get("FacebookScreenName"))
        instagram_screen_name = from_union([from_none, from_str], obj.get("InstagramScreenName"))
        instagram_profile_url = from_union([from_none, from_str], obj.get("InstagramProfileUrl"))
        karma = from_union([from_none, Karma.from_dict], obj.get("Karma"))
        curse_period = from_union([from_none, CursePeriod.from_dict], obj.get("CursePeriod"))
        return UserInfo(user_identifier, remaining_invitation, entry_counts, last_entry_date, standing_queue_number, has_any_pending_invitation, is_buddy, is_blocked, is_followed, is_corporate, is_deactivated, is_karma_shown, is_caylak, is_index_titles_blocked, note, is_cursed, is_banned, display_twitter_profile, display_facebook_profile, display_instagram_profile, twitter_screen_name, facebook_profile_url, facebook_screen_name, instagram_screen_name, instagram_profile_url, karma, curse_period)

    def to_dict(self) -> dict:
        result: dict = {}
        result["UserIdentifier"] = to_class(UserIdentifier, self.user_identifier)
        result["RemainingInvitation"] = from_int(self.remaining_invitation)
        result["EntryCounts"] = to_class(UserInfoEntryCounts, self.entry_counts)
        result["LastEntryDate"] = self.last_entry_date.isoformat()
        result["StandingQueueNumber"] = from_int(self.standing_queue_number)
        result["HasAnyPendingInvitation"] = from_bool(self.has_any_pending_invitation)
        result["IsBuddy"] = from_bool(self.is_buddy)
        result["IsBlocked"] = from_bool(self.is_blocked)
        result["IsFollowed"] = from_bool(self.is_followed)
        result["IsCorporate"] = from_bool(self.is_corporate)
        result["IsDeactivated"] = from_bool(self.is_deactivated)
        result["IsKarmaShown"] = from_bool(self.is_karma_shown)
        result["IsCaylak"] = from_bool(self.is_caylak)
        result["IsIndexTitlesBlocked"] = from_bool(self.is_index_titles_blocked)
        result["Note"] = from_none(self.note)
        result["IsCursed"] = from_bool(self.is_cursed)
        result["IsBanned"] = from_bool(self.is_banned)
        result["DisplayTwitterProfile"] = from_bool(self.display_twitter_profile)
        result["DisplayFacebookProfile"] = from_bool(self.display_facebook_profile)
        result["DisplayInstagramProfile"] = from_bool(self.display_instagram_profile)
        result["TwitterScreenName"] = from_union([from_none, from_str], self.twitter_screen_name)
        result["FacebookProfileUrl"] = from_union([from_none, from_str], self.facebook_profile_url)
        result["FacebookScreenName"] = from_union([from_none, from_str], self.facebook_screen_name)
        result["InstagramScreenName"] = from_union([from_none, from_str], self.instagram_screen_name)
        result["InstagramProfileUrl"] = from_union([from_none, from_str], self.instagram_profile_url)
        result["Karma"] = from_union([from_none, lambda x: to_class(Karma, x)], self.karma)
        result["CursePeriod"] = from_union([from_none, lambda x: to_class(CursePeriod, x)], self.curse_period)
        return result


from eksisozluk.Models.Responses.ResponseMessage import Message
from eksisozluk.Models.User.EntrySummary import EntrySummary
from eksisozluk.Models.__init__ import *


@dataclass
class UserEntries:
    pinned_entry: None
    entries: List[EntrySummary]
    page_count: int
    page_size: int
    page_index: int

    @staticmethod
    def from_dict(obj: Any) -> 'UserEntries':
        assert isinstance(obj, dict)
        pinned_entry = from_none(obj.get("PinnedEntry"))
        entries = from_list(EntrySummary.from_dict, obj.get("Entries"))
        page_count = from_int(obj.get("PageCount"))
        page_size = from_int(obj.get("PageSize"))
        page_index = from_int(obj.get("PageIndex"))
        return UserEntries(pinned_entry, entries, page_count, page_size, page_index)

    def to_dict(self) -> dict:
        result: dict = {}
        result["PinnedEntry"] = from_none(self.pinned_entry)
        result["Entries"] = from_list(lambda x: to_class(EntrySummary, x), self.entries)
        result["PageCount"] = from_int(self.page_count)
        result["PageSize"] = from_int(self.page_size)
        result["PageIndex"] = from_int(self.page_index)
        return result


@dataclass
class UserEntriesResponse:
    success: bool
    data: UserEntries
    message: Optional[Message] = None

    @staticmethod
    def from_dict(obj: Any) -> 'UserEntriesResponse':
        assert isinstance(obj, dict)
        success = from_bool(obj.get("Success"))
        message = from_union([from_none, Message], obj.get("Message"))
        data = UserEntries.from_dict(obj.get("Data"))
        return UserEntriesResponse(success, message, data)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Success"] = from_bool(self.success)
        result["Message"] = from_union([from_none, lambda x: to_enum(Message, x)], self.message)
        result["Data"] = to_class(UserEntries, self.data)
        return result


def user_entries_response_from_dict(s: Any) -> UserEntriesResponse:
    return UserEntriesResponse.from_dict(s)


def user_entries_response_to_dict(x: UserEntriesResponse) -> Any:
    return to_class(UserEntriesResponse, x)



from dataclasses import dataclass
from datetime import datetime
from typing import Any

from eksisozluk.Models import from_datetime


@dataclass
class CursePeriod:
    curse_period_from: datetime
    to: datetime

    @staticmethod
    def from_dict(obj: Any) -> 'CursePeriod':
        assert isinstance(obj, dict)
        curse_period_from = from_datetime(obj.get("From"))
        to = from_datetime(obj.get("To"))
        return CursePeriod(curse_period_from, to)

    def to_dict(self) -> dict:
        result: dict = {}
        result["From"] = self.curse_period_from.isoformat()
        result["To"] = self.to.isoformat()
        return result


from dataclasses import dataclass, field
from datetime import datetime
from typing import Any, TypeVar, Type, cast, Optional
from dataclasses import dataclass
from typing import Any, Optional, List, TypeVar, Type, cast, Callable
from datetime import datetime
from enum import Enum
import dateutil.parser

T = TypeVar("T")
EnumT = TypeVar("EnumT", bound=Enum)


def from_none(x: Any) -> Any:
    assert x is None
    return x


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_bool(x: Any) -> bool:
    assert isinstance(x, bool)
    return x


def from_int(x: Any) -> int:
    assert isinstance(x, int)
    return x


def from_union(fs, x):
    for f in fs:
        try:
            return f(x)
        except:
            pass
    assert False


def from_datetime(x: Any) -> datetime:
    return dateutil.parser.parse(x)


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def to_enum(c: Type[EnumT], x: Any) -> EnumT:
    assert isinstance(x, c)


def is_type(t: Type[T], x: Any) -> T:
    assert isinstance(x, t)
    return x


def parse_date(date_string: str) -> datetime:
    return datetime.strptime(date_string, "%m/%d/%Y %I:%M:%S %p %z")



from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import from_str, from_int


@dataclass
class UserIdentifier:
    nick: str
    id: int

    @staticmethod
    def from_dict(obj: Any) -> 'UserIdentifier':
        assert isinstance(obj, dict)
        nick = from_str(obj.get("Nick"))
        id = from_int(obj.get("Id"))
        return UserIdentifier(nick, id)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Nick"] = from_str(self.nick)
        result["Id"] = from_int(self.id)
        return result


import uuid
import requests
import logging
from eksisozluk.Models.Auth.EksiToken import EksiToken
from eksisozluk.Models.Entry.Entry import Entry
from eksisozluk.Models.Exceptions.ClientException import ClientException
from eksisozluk.Models.Exceptions.EntryNotFoundException import EntryNotFoundException
from eksisozluk.Models.Exceptions.TopicNotFoundException import TopicNotFoundException
from eksisozluk.Models.Exceptions.UserNotFoundException import UserNotFoundException
from eksisozluk.Models.Responses.IndexResponse import IndexResponse
from eksisozluk.Models.Responses.LoginResponse import LoginResponse
from eksisozluk.Models.Responses.TopicResponse import TopicResponse
from eksisozluk.Models.Responses.ResponseMessage import Message
from eksisozluk.Models.Responses.UserEntriesResponse import UserEntriesResponse
from eksisozluk.Models.Responses.UserResponse import UserResponse
from eksisozluk.Models.Topic.Topic import Topic
from eksisozluk.Models.User.User import User

logging.basicConfig(level=logging.DEBUG)

api = "https://api.eksisozluk.com"
api_secret = "68f779c5-4d39-411a-bd12-cbcc50dc83dd"

routes = {
    "login": "/Token",
    "anon_login": "/v2/account/anonymoustoken",
    "client_info": "/v2/clientsettings/info",
    "topic": "/v2/topic/{}",
    "entry": "/v2/entry/{}",
    "entry_favorite": "/v2/entry/favorite",
    "entry_unfavorite": "/v2/entry/unfavorite",
    "user": "/v2/user/{}",
    "user_follow": "/v2/user/follow",
    "user_unfollow": "/v2/user/unfollow",
    "user_block": "/v2/user/block",
    "user_unblock": "/v2/user/unblock",
    "user_indextitlesblock": "/v2/user/indextitlesblock",
    "user_removeindextitlesblock": "/v2/user/indextitlesblock",
    "user_entries": "/v2/user/{}/entries",
    "user_favorited": "/v2/user/{}/favorited",
    "user_lastvoted": "/v2/user/{}/lastvoted",
    "user_lastweekmostvoted": "/v2/user/{}/lastweekmostvoted",
    "index_popular": "/v2/index/popular",
    "index_today": "/v2/index/today",
    "index_getuserchannelfilters": "/v2/index/getuserchannelfilters",
    "index_setchannelfilter": "/v2/index/setchannelfilter",
}


class EksiApi:
    def __init__(self, username: str = None, password: str = None) -> None:
        self.username = username
        self.password = password
        self.token = None
        self.client_secret = str(uuid.uuid1())
        self.client_unique_id = str(uuid.uuid1())
        self.session = requests.session()
        self.session.hooks = dict(response=self.__hook)
        self.__logger = logging.getLogger(__name__)
        self.__logger.setLevel(logging.DEBUG)
        self.auth()

    def set_session_headers(self):
        self.session.headers["User-agent"] = "okhttp/3.12.1"
        self.session.headers["Authorization"] = "Bearer " + self.token.access_token
        self.session.headers["Client-Secret"] = self.client_secret
        self.session.headers["Api-Secret"] = api_secret

    def get_client_info(self):
        headers = {
            "Client-secret": self.client_secret,
            "Content-type": "application/x-www-form-urlencoded",
            "User-agent": "okhttp/3.12.1"
        }
        payload = {
            "Api-secret": api_secret
        }
        response = requests.post(api + routes["client_info"], headers=headers, data=payload)
        return response.json()

    def __hook(self, res, *args, **kwargs):
        if res.status_code == requests.codes.unauthorized:
            self.__logger.info("Token expired, refreshing")
            self.auth()
            req = res.request
            self.__logger.info("Resending request", req.method, req.url, req.headers)
            req.headers["Authorization"] = self.session.headers["Authorization"]
            return self.session.send(res.request)

    def auth(self):
        if self.username and self.password:
            self.token = self.login(self.username, self.password)
        else:
            self.token = self.anon_login()
        self.__logger.debug("Token: {}".format(self.token))
        self.set_session_headers()

    def anon_login(self) -> EksiToken:
        url = api + routes["anon_login"]
        payload = {
            "Platform": "g",
            "Version": "2.0.0",
            "Build": "51",
            "Api-Secret": api_secret,
            "Client-Secret": self.client_secret,
            "ClientUniqueId": self.client_unique_id
        }
        headers = {
            "Content-type": "application/x-www-form-urlencoded",
            "User-agent": "okhttp/3.12.1"
        }
        response = requests.post(url, headers=headers, data=payload)
        return LoginResponse.from_dict(response.json()).data

    def login(self, username: str, password: str) -> EksiToken:
        url = api + routes["login"]
        payload = {
            "password": password,
            "Platform": "g",
            "Version": "2.0.0",
            "grant_type": "password",
            "Build": "51",
            "Api-Secret": api_secret,
            "Client-Secret": self.client_secret,
            "ClientUniqueId": self.client_unique_id,
            "username": username
        }
        headers = {
            "Content-type": "application/x-www-form-urlencoded",
            "User-Agent": "okhttp/3.12.1"
        }
        response = requests.post(url, headers=headers, data=payload)
        return EksiToken.from_dict(response.json())

    def get_entry(self, entry_id: int) -> Entry:
        url = api + routes["entry"].format(entry_id)
        response = self.session.get(url)
        topic_response = TopicResponse.from_dict(response.json())
        if topic_response.message == Message.ENTRY_BULUNAMADI:
            raise EntryNotFoundException("Entry not found")
        return topic_response.topic.get_first_entry()

    def get_topic(self, topic_id: int, page=1) -> TopicResponse:
        url = api + routes["topic"].format(topic_id) + "?p={}".format(page)
        response = self.session.get(url)
        if response.status_code == 200:
            return TopicResponse.from_dict(response.json())
        elif response.status_code == 500:
            raise TopicNotFoundException("Topic not found")
        raise ClientException(f"Unknown error: {response.status_code}")

    def get_user(self, user_nick: str) -> User:
        url = api + routes["user"].format(user_nick)
        response = self.session.get(url)
        if response.status_code == 200:
            user_response = UserResponse.from_dict(response.json())
            return user_response.user
        elif response.status_code == 500:
            raise UserNotFoundException("User not found")
        raise ClientException(f"Unknown error: {response.status_code}")

    def get_user_entries(self, user_nick: str, page=1) -> UserEntriesResponse:
        url = api + routes["user_entries"] + "?p={}".format(page)
        response = self.session.get(url.format(user_nick))
        if response.status_code == 200:
            return UserEntriesResponse.from_dict(response.json())
        raise ClientException(f"Unknown error: {response.status_code}")

    def get_user_favorites(self, user_nick: str, page=1) -> dict:
        url = api + routes["user_favorited"] + "?p={}".format(page)
        response = self.session.get(url.format(user_nick))
        return response.json()

    def get_user_lastvoted(self, user_nick: str, page=1) -> dict:
        url = api + routes["user_lastvoted"] + "?p={}".format(page)
        response = self.session.get(url.format(user_nick))
        return response.json()

    def follow_user(self, user_nick: str) -> dict:
        url = api + routes["user_follow"]
        response = self.session.post(url, data={"user_nick": user_nick})
        return response.json()

    def unfollow_user(self, user_nick: str) -> dict:
        url = api + routes["user_unfollow"]
        response = self.session.post(url, data={"user_nick": user_nick})
        return response.json()

    def block_user(self, user_nick: str) -> dict:
        url = api + routes["user_block"]
        response = self.session.post(url, data={"user_nick": user_nick})
        return response.json()

    def unblock_user(self, user_nick: str) -> dict:
        url = api + routes["user_unblock"]
        response = self.session.post(url, data={"user_nick": user_nick})
        return response.json()

    def get_index_filters(self) -> dict:
        url = api + routes["index_getuserchannelfilters"]
        response = self.session.get(url)
        return response.json()

    def set_index_filter(self, filters: dict) -> dict:
        url = api + routes["index_setchannelfilter"]
        response = self.session.post(url, json=filters)
        return response.json()

    def get_index_popular(self, page=1) -> IndexResponse:
        url = api + routes["index_popular"] + "?p={}".format(page)
        response = self.session.get(url)
        if response.status_code == 200:
            return IndexResponse.from_dict(response.json())
        raise ClientException(f"Unknown error: {response.status_code}")

    def get_index_today(self, page=1) -> IndexResponse:
        url = api + routes["index_today"] + "?p={}".format(page)
        response = self.session.get(url)
        if response.status_code == 200:
            return IndexResponse.from_dict(response.json())
        raise ClientException(f"Unknown error: {response.status_code}")

    def favorite_entry(self, entry_id: int) -> dict:
        url = api + routes["entry_favorite"]
        response = self.session.post(url, data={"entry_id": entry_id})
        return response.json()

    def unfavorite_entry(self, entry_id: int) -> dict:
        url = api + routes["entry_unfavorite"]
        response = self.session.delete(url, data={"entry_id": entry_id})
        return response.json()



from dataclasses import dataclass
from datetime import datetime
from typing import Optional, List, Any

from eksisozluk.Models import from_int, from_str, from_datetime, from_bool, from_none, from_union, from_list, to_class
from eksisozluk.Models.User.UserIdentifier import UserIdentifier


@dataclass
class Entry:
    id: int
    content: str
    author: UserIdentifier
    created: datetime
    is_favorite: bool
    favorite_count: int
    hidden: bool
    active: bool
    comment_count: int
    comment_summary: None
    last_updated: Optional[datetime] = None
    avatar_url: Optional[str] = None
    media: Optional[List[str]] = None

    @staticmethod
    def from_dict(obj: Any) -> 'Entry':
        assert isinstance(obj, dict)
        id = from_int(obj.get("Id"))
        content = from_str(obj.get("Content"))
        author = UserIdentifier.from_dict(obj.get("Author"))
        created = from_datetime(obj.get("Created"))
        is_favorite = from_bool(obj.get("IsFavorite"))
        favorite_count = from_int(obj.get("FavoriteCount"))
        hidden = from_bool(obj.get("Hidden"))
        active = from_bool(obj.get("Active"))
        comment_count = from_int(obj.get("CommentCount"))
        comment_summary = from_none(obj.get("CommentSummary"))
        last_updated = from_union([from_none, from_datetime], obj.get("LastUpdated"))
        avatar_url = from_union([from_none, from_str], obj.get("AvatarUrl"))
        media = from_union([from_none, lambda x: from_list(from_str, x)], obj.get("Media"))
        return Entry(id, content, author, created, is_favorite, favorite_count, hidden, active, comment_count, comment_summary, last_updated, avatar_url, media)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Id"] = from_int(self.id)
        result["Content"] = from_str(self.content)
        result["Author"] = to_class(UserIdentifier, self.author)
        result["Created"] = self.created.isoformat()
        result["IsFavorite"] = from_bool(self.is_favorite)
        result["FavoriteCount"] = from_int(self.favorite_count)
        result["Hidden"] = from_bool(self.hidden)
        result["Active"] = from_bool(self.active)
        result["CommentCount"] = from_int(self.comment_count)
        result["CommentSummary"] = from_none(self.comment_summary)
        result["LastUpdated"] = from_union([from_none, lambda x: x.isoformat()], self.last_updated)
        result["AvatarUrl"] = from_union([from_none, from_str], self.avatar_url)
        result["Media"] = from_union([from_none, lambda x: from_list(from_str, x)], self.media)
        return result


from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import from_int


@dataclass
class PinnedEntryEntryCounts:
    before_first_entry: int
    after_last_entry: int
    buddy: int
    total: int

    @staticmethod
    def from_dict(obj: Any) -> 'PinnedEntryEntryCounts':
        assert isinstance(obj, dict)
        before_first_entry = from_int(obj.get("BeforeFirstEntry"))
        after_last_entry = from_int(obj.get("AfterLastEntry"))
        buddy = from_int(obj.get("Buddy"))
        total = from_int(obj.get("Total"))
        return PinnedEntryEntryCounts(before_first_entry, after_last_entry, buddy, total)

    def to_dict(self) -> dict:
        result: dict = {}
        result["BeforeFirstEntry"] = from_int(self.before_first_entry)
        result["AfterLastEntry"] = from_int(self.after_last_entry)
        result["Buddy"] = from_int(self.buddy)
        result["Total"] = from_int(self.total)
        return result


from enum import Enum


class KarmaName(Enum):
    AKLISELIM = "aklıselim"
    ANADOLU_ÇOCUĞU = "anadolu çocuğu"
    ANARŞIST = "anarşist"
    AĞIR_ABI = "ağır abi"
    BALDAN_TATLI = "baldan tatlı"
    BAL_KÜPÜ = "bal küpü"
    BATTAL_GAZI = "battal gazi"
    BIÇKIN = "bıçkın"
    DELIKANLI = "delikanlı"
    DÜZ_ADAM = "düz adam"
    ENERGIZER_TAVŞANI = "energizer tavşanı"
    GENÇLERIN_SEVGILISI = "gençlerin sevgilisi"
    HER_EVE_LAZIM = "her eve lazım"
    HIPPI = "hippi"
    HIRÇIN_GOLCÜ = "hırçın golcü"
    IÇ_GÜVEYSINDEN_HALLICE = "iç güveysinden hallice"
    KAOTIK_NÖTRAL = "kaotik nötral"
    KIZGIN_KUMLARDAN_SERIN_SULARA = "kızgın kumlardan serin sulara"
    KOFTI_ANARŞIST = "kofti anarşist"
    LEZIZ = "leziz"
    MANGAL_YÜREKLI_RIŞAR = "mangal yürekli rişar"
    MINYONLARIN_SEVGILISI = "minyonların sevgilisi"
    MÜLAYIM_AMA_SEMPATIK = "mülayim ama sempatik"
    MÜZMIN_YEDEK = "müzmin yedek"
    PADAWAN = "padawan"
    PREZENTABL = "prezentabl"
    RATING_CANAVARI = "rating canavarı"
    TADINA_DOYUM_OLMAZ = "tadına doyum olmaz"
    ÇETREFILLI = "çetrefilli"
    ÇILGIN = "çılgın"
    ŞAMDA_KAYISI = "şamda kayısı"
    ŞEKERPARE = "şekerpare"
    ŞEKER_ABI = "şeker abi"


from dataclasses import dataclass
from datetime import datetime
from typing import Any

from eksisozluk.Models import from_datetime, from_int, from_str


@dataclass
class IndexedTopic:
    day: datetime
    matched_count: int
    topic_id: int
    full_count: int
    title: str

    @staticmethod
    def from_dict(obj: Any) -> 'IndexedTopic':
        assert isinstance(obj, dict)
        day = from_datetime(obj.get("Day"))
        matched_count = from_int(obj.get("MatchedCount"))
        topic_id = from_int(obj.get("TopicId"))
        full_count = from_int(obj.get("FullCount"))
        title = from_str(obj.get("Title"))
        return IndexedTopic(day, matched_count, topic_id, full_count, title)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Day"] = self.day.isoformat()
        result["MatchedCount"] = from_int(self.matched_count)
        result["TopicId"] = from_int(self.topic_id)
        result["FullCount"] = from_int(self.full_count)
        result["Title"] = from_str(self.title)
        return result


from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import to_class
from eksisozluk.Models.Entry.Entry import Entry
from eksisozluk.Models.User.TopicID import TopicID


@dataclass
class EntrySummary:
    topic_id: TopicID
    entry: Entry

    @staticmethod
    def from_dict(obj: Any) -> 'EntrySummary':
        assert isinstance(obj, dict)
        topic_id = TopicID.from_dict(obj.get("TopicId"))
        entry = Entry.from_dict(obj.get("Entry"))
        return EntrySummary(topic_id, entry)

    def to_dict(self) -> dict:
        result: dict = {}
        result["TopicId"] = to_class(TopicID, self.topic_id)
        result["Entry"] = to_class(Entry, self.entry)
        return result


from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import from_int, from_str, to_class
from eksisozluk.Models.User.TopicTitle import TopicTitle


@dataclass
class TopicID:
    id: int
    topic_title: TopicTitle
    title: str

    @staticmethod
    def from_dict(obj: Any) -> 'TopicID':
        assert isinstance(obj, dict)
        id = from_int(obj.get("Id"))
        topic_title = TopicTitle.from_dict(obj.get("TopicTitle"))
        title = from_str(obj.get("Title"))
        return TopicID(id, topic_title, title)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Id"] = from_int(self.id)
        result["TopicTitle"] = to_class(TopicTitle, self.topic_title)
        result["Title"] = from_str(self.title)
        return result


from eksisozluk.Models.Responses.ResponseMessage import Message
from eksisozluk.Models.Topic.Topic import Topic
from eksisozluk.Models.__init__ import *


@dataclass
class TopicResponse:
    success: Optional[bool] = None
    message: Optional[Message] = None
    topic: Optional[Topic] = None

    @staticmethod
    def from_dict(obj: Any) -> 'TopicResponse':
        assert isinstance(obj, dict)
        success = from_union([from_bool, from_none], obj.get("Success"))
        message = from_union([from_none, Message], obj.get("Message"))
        data = from_union([Topic.from_dict, from_none], obj.get("Data"))
        return TopicResponse(success, message, data)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Success"] = from_union([from_bool, from_none], self.success)
        result["Message"] = from_union([from_none, lambda x: to_enum(Message, x)], self.message)
        result["Data"] = from_union([lambda x: to_class(Topic, x), from_none], self.topic)
        return result


def topic_response_from_dict(s: Any) -> List[TopicResponse]:
    return from_list(TopicResponse.from_dict, s)


def topic_response_to_dict(x: List[TopicResponse]) -> Any:
    return from_list(lambda x: to_class(TopicResponse, x), x)



import pytest

from eksisozluk.EksiSozluk import EksiApi

api: EksiApi


@pytest.mark.dependency()
def test_anon_client():
    global api
    api = EksiApi()
    assert api.token is not None


@pytest.mark.dependency(depends=['test_anon_client'])
def test_anon_client_get_entry():
    entry = api.get_entry(1)
    assert (entry.id == 1)


@pytest.mark.dependency(depends=['test_anon_client'])
def test_anon_client_get_topic():
    topic = api.get_topic(7006370)
    assert topic.success


@pytest.mark.dependency(depends=['test_anon_client'])
def test_anon_client_get_user():
    user = api.get_user("nimzo")
    assert user.user_info.user_identifier.nick == "nimzo"

@pytest.mark.dependency(depends=['test_anon_client'])
def test_anon_client_get_index_today():
    topic_response = api.get_index_today()
    assert len(topic_response.index.topics) > 0



from dataclasses import dataclass
from typing import List, Any

from eksisozluk.Models import from_list, from_int, from_bool, from_none, to_class
from eksisozluk.Models.Index.IndexedTopic import IndexedTopic


@dataclass
class Index:
    topics: List[IndexedTopic]
    page_count: int
    page_size: int
    page_index: int
    has_pinned_index_item: bool
    pinned_index_item: None

    @staticmethod
    def from_dict(obj: Any) -> 'Index':
        assert isinstance(obj, dict)
        topics = from_list(IndexedTopic.from_dict, obj.get("Topics"))
        page_count = from_int(obj.get("PageCount"))
        page_size = from_int(obj.get("PageSize"))
        page_index = from_int(obj.get("PageIndex"))
        has_pinned_index_item = from_bool(obj.get("HasPinnedIndexItem"))
        pinned_index_item = from_none(obj.get("PinnedIndexItem"))
        return Index(topics, page_count, page_size, page_index, has_pinned_index_item, pinned_index_item)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Topics"] = from_list(lambda x: to_class(IndexedTopic, x), self.topics)
        result["PageCount"] = from_int(self.page_count)
        result["PageSize"] = from_int(self.page_size)
        result["PageIndex"] = from_int(self.page_index)
        result["HasPinnedIndexItem"] = from_bool(self.has_pinned_index_item)
        result["PinnedIndexItem"] = from_none(self.pinned_index_item)
        return result


from dataclasses import dataclass
from typing import Any

from eksisozluk.Models import from_str, from_none


@dataclass
class TopicTitle:
    title: str
    kind: None

    @staticmethod
    def from_dict(obj: Any) -> 'TopicTitle':
        assert isinstance(obj, dict)
        title = from_str(obj.get("Title"))
        kind = from_none(obj.get("Kind"))
        return TopicTitle(title, kind)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Title"] = from_str(self.title)
        result["Kind"] = from_none(self.kind)
        return result


from dataclasses import dataclass
from typing import List, Any, Optional

from eksisozluk.Models import from_int, from_str, from_list, from_none, from_bool, from_union, to_class
from eksisozluk.Models.Entry.Entry import Entry
from eksisozluk.Models.User.EntryCounts import EntryCounts
from eksisozluk.Models.Topic.Video import Video


@dataclass
class Topic:
    id: int
    title: str
    entries: List[Entry]
    page_count: int
    page_size: int
    page_index: int
    pinned_entry: None
    entry_counts: EntryCounts
    draft_entry: None
    is_tracked: bool
    is_trackable: bool
    disambiguations: List[Any]
    is_ama_topic: bool
    matter_count: int
    slug: Optional[str] = None
    video: Optional[Video] = None

    def get_first_entry(self) -> Optional[Entry]:
        if len(self.entries) > 0:
            return self.entries[0]
        return None

    @staticmethod
    def from_dict(obj: Any) -> 'Topic':
        assert isinstance(obj, dict)
        id = from_int(obj.get("Id"))
        title = from_str(obj.get("Title"))
        entries = from_list(Entry.from_dict, obj.get("Entries"))
        page_count = from_int(obj.get("PageCount"))
        page_size = from_int(obj.get("PageSize"))
        page_index = from_int(obj.get("PageIndex"))
        pinned_entry = from_none(obj.get("PinnedEntry"))
        entry_counts = EntryCounts.from_dict(obj.get("EntryCounts"))
        draft_entry = from_none(obj.get("DraftEntry"))
        is_tracked = from_bool(obj.get("IsTracked"))
        is_trackable = from_bool(obj.get("IsTrackable"))
        disambiguations = from_list(lambda x: x, obj.get("Disambiguations"))
        is_ama_topic = from_bool(obj.get("IsAmaTopic"))
        matter_count = from_int(obj.get("MatterCount"))
        slug = from_union([from_none, from_str], obj.get("Slug"))
        video = from_union([Video.from_dict, from_none], obj.get("Video"))
        return Topic(id, title, entries, page_count, page_size, page_index, pinned_entry, entry_counts, draft_entry, is_tracked, is_trackable, disambiguations, is_ama_topic, matter_count, slug, video)

    def to_dict(self) -> dict:
        result: dict = {}
        result["Id"] = from_int(self.id)
        result["Title"] = from_str(self.title)
        result["Entries"] = from_list(lambda x: to_class(Entry, x), self.entries)
        result["PageCount"] = from_int(self.page_count)
        result["PageSize"] = from_int(self.page_size)
        result["PageIndex"] = from_int(self.page_index)
        result["PinnedEntry"] = from_none(self.pinned_entry)
        result["EntryCounts"] = to_class(EntryCounts, self.entry_counts)
        result["DraftEntry"] = from_none(self.draft_entry)
        result["IsTracked"] = from_bool(self.is_tracked)
        result["IsTrackable"] = from_bool(self.is_trackable)
        result["Disambiguations"] = from_list(lambda x: x, self.disambiguations)
        result["IsAmaTopic"] = from_bool(self.is_ama_topic)
        result["MatterCount"] = from_int(self.matter_count)
        result["Slug"] = from_union([from_none, from_str], self.slug)
        result["Video"] = from_union([lambda x: to_class(Video, x), from_none], self.video)
        return result


from eksisozluk.Models import *
from datetime import datetime, timedelta


@dataclass
class EksiToken:
    rank: int
    access_token: str
    token_type: str
    expires_in: int
    user_id: Optional[int] = None
    refresh_token: Optional[str] = None
    nick: Optional[str] = None
    issued_at: Optional[datetime] = None
    expires_at: Optional[datetime] = None

    def __post_init__(self):
        self.issued_at = datetime.utcnow()
        self.expires_at = self.issued_at + timedelta(seconds=self.expires_in)

    def is_user(self) -> bool:
        assert(self.is_valid)
        return self.user_id is not None

    def is_valid(self) -> bool:
        return datetime.utcnow() < self.expires_at

    @staticmethod
    def from_dict(obj: Any) -> 'EksiToken':
        assert isinstance(obj, dict)
        rank = int(from_str(obj.get("rank")))
        access_token = from_str(obj.get("access_token"))
        token_type = from_str(obj.get("token_type"))
        expires_in = from_union([from_int, lambda x: int(from_str(x))], obj.get("expires_in"))
        user_id = from_union([from_none, lambda x: int(from_str(x))], obj.get("user_id"))
        refresh_token = from_union([from_str, from_none], obj.get("refresh_token"))
        nick = from_union([from_str, from_none], obj.get("nick"))
        return EksiToken(rank, access_token, token_type, expires_in, user_id, refresh_token, nick)

    def to_dict(self) -> dict:
        result: dict = {}
        result["rank"] = from_str(str(self.rank))
        result["access_token"] = from_str(self.access_token)
        result["token_type"] = from_str(self.token_type)
        result["expires_in"] = from_int(self.expires_in)
        result["user_id"] = from_union([lambda x: from_none((lambda x: is_type(type(None), x))(x)), lambda x: from_str((lambda x: str((lambda x: is_type(int, x))(x)))(x))], self.user_id)
        result["refresh_token"] = from_union([from_str, from_none], self.refresh_token)
        result["nick"] = from_union([from_str, from_none], self.nick)
        return result


